function [at,ct,pva,pvb,pwa,pwb]=arrangeA(AT,CT,PvT,PwT)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The input is a system of the form x(t)=AT*x(t-1)+CT+PvT*v(t)+PwT*w(t)
% The output is a system 
% x'(t)= at*x'(t-1) + ct + pav*v(t) + pbv*v(t-1)  + paw*w(t) 
%        + pbw*w(t-1) + pcw*w(t-2)
% AT is assumed to have n2 zero eigenvalues 
% n1=n-n2 is the number of non-zero-eigenvalues
%
% This program uses the m-files qzzero and qzswitch by Chris Sims, see
% http://eco-072399b.princeton.edu/yftp/gensys/mfiles/QZSWITCH.M
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Andreas Beyer and Roger E. A. Farmer
% February 2004: Amended November 2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Corrected version of arrange
% Bug fixed June 28th 2004 RF
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

eps=1E-10;
n=size(AT,1);
%%    next line corrected to add absolute value operator [RF  June 28th
%%    2004]
nnz=sum(abs(eig(AT))>eps);   % This is the number of non-zero eigenvalues of AT
nz=n-nnz;               % This is the number of zero eigenvalues of AT
D=eye(n);
[D, T, Q, Z]=qz(D,AT);

% This next line rearranges the matrices D T Q and Z 
% so that all zeros eigenvalues of T  appear on the lower right diagonal
% while keeping Q'DZ' and Q'TZ' unchanged
[D,T, Q, Z]= qzzero(D,T,Q,Z);   

% This checks to make sure that D is diagonal
% Next lines corrected to add absolute value operator [RF June 28th 2004]
D12=D(1:nnz,nnz+1:n);D21=D(nnz+1:n,1:nnz);
D12=sum(sum(abs(D12)));D21=sum(sum(abs(D21)));
if or(abs(D12)>eps,abs(D21)>eps);
    disp('Error: D is not diagonal');
    return
end

DI=inv(D);
S=DI*T;
C=DI*Q*CT;Pv=DI*Q*PvT;Pw=DI*Q*PwT;
C1=C(1:nnz);C2=C(nnz+1:n);
Pv1=Pv(1:nnz,:);Pv2=Pv(nnz+1:n,:);
if ~isempty(Pw)
Pw1=Pw(1:nnz,:);Pw2=Pw(nnz+1:n,:);
else Pw1=[];Pw2=[];
end;
s11=S(1:nnz,1:nnz);s12=S(1:nnz,nnz+1:n);
s21=S(nnz+1:n,1:nnz);s22=S(nnz+1:n,nnz+1:n);

% Construct partition of Z'
ZT=Z';
z11=ZT(1:nnz,1:nnz);z12=ZT(1:nnz,nnz+1:n);
z21=ZT(nnz+1:n,1:nnz);z22=ZT(nnz+1:n,nnz+1:n);

N1=-inv(z22)*z21;N2=inv(z22);N3=inv(z22)*s22;
M2=inv(z11+z12*N1);M1=M2*s11*(z11+z12*N1);
M3=-M2*z12*N2;M4=M2*(s11*z12*N2+s12-z12*N3);
M5=M2*(s11*z12*N3+s12*s22);


at=real([M1 zeros(nnz,nz); N1*M1 zeros(nz,nz)]);
ct=real([M2*C1+(M3+M4+M5)*C2;N1*(M2*C1+(M3+M4+M5)*C2)+(N2+N3)*C2]);
pva=real([M2*Pv1+M3*Pv2;N1*(M2*Pv1+M3*Pv2)+N2*Pv2]);
pvb=real([M4*Pv2;N1*(M4*Pv2)+N3*Pv2]);
% pvc=real([M5*Pv2;N1*(M5*Pv2)]); Removed November 2004
if ~isempty(PwT); 
    pwa=real([M2*Pw1+M3*Pw2;N1*(M2*Pw1+M3*Pw2)+N2*Pw2]);
    pwb=real([M4*Pw2;N1*(M4*Pw2)+N3*Pw2]);
%    pwc=real([M5*Pw2;N1*(M5*Pw2)]); Removed November 2004
else
    pwa=[];
    pwb=[];
%    pwc=[];  Removed November 2004
end
pvt=0;pwt=0;pvLt=0;pwLt=0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  END %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%